// File: Myhash.h

#ifndef HASH_H
#define HASH_H

#include "list.h"
#include "Duplicate_error.h"
#include <iostream>

template <typename T, unsigned buckets = 2000 >
class Myhash
{
    List<T> array[buckets];
    // hash function borrowed from: Thomas Wang https://gist.github.com/badboy/6267743
    unsigned hash(unsigned key) const
    {
        int c2 = 0x27d4eb2d; // a prime or an odd constant
        key = (key ^ 61) ^ (key >> 16);
        key = key + (key << 3);
        key = key ^ (key >> 4);
        key = key * c2;
        key = key ^ (key >> 15);
        return key % buckets;
    }
    unsigned size_; // number of values stored in the hash table
    bool throwIfDuplicate;
public:
    Myhash();

    unsigned size() const
    {
        return size_;
    }
    bool insert(T value);
    bool find(T value) const;
    unsigned bucketSize(unsigned bucket) const;
    float percentOfBucketsUsed() const;
    float averageNonEmptyBucketSize() const;
    unsigned largestBucket() const;
    unsigned largestBucketSize() const;
};

template <typename T, unsigned buckets>
Myhash<T, buckets>::Myhash() : size_(0)
{
}

template <typename T, unsigned buckets>
bool Myhash<T, buckets>::insert(T value)
{
    if (find(value)) {
        if (throwIfDuplicate) throw DuplicateError<T>(value);
    }
    unsigned convert = value;
    array[hash(convert)].push(value);
    size_++;
    return true;
}

template <typename T, unsigned buckets>
bool Myhash<T, buckets>::find(T value) const
{
    unsigned convert = value;
    return array[hash(convert)].find(value) != 0;
}

template <typename T, unsigned buckets>
unsigned Myhash<T, buckets>::bucketSize(unsigned bucket) const
{
    return array[bucket].size();
}

template <typename T, unsigned buckets>
float Myhash<T, buckets>::averageNonEmptyBucketSize() const
{
    unsigned nonEmptyBuckets = 0;
    float sum = 0;
    for (unsigned i = 0; i < buckets; i++) {
        if (bucketSize(i)) {
            nonEmptyBuckets++;
            sum += bucketSize(i);
        }
    }
    return sum / buckets;
}

template <typename T, unsigned buckets>
float Myhash<T, buckets>::percentOfBucketsUsed() const
{
    static ofstream fout("bucket_contents.txt");
    unsigned numberOfBucketsUsed = 0;
    for (unsigned i = 0; i < buckets; i++) {
            fout << i << ' ' << bucketSize(i) << endl;
        if (bucketSize(i)) numberOfBucketsUsed++;
    }
    return static_cast<float> (numberOfBucketsUsed) / buckets;
}

template <typename T, unsigned buckets>
unsigned Myhash<T, buckets>::largestBucket() const
{
    unsigned largestBucket = 0;
    unsigned largestBucketSize = 0;
    for (unsigned i = 0; i < buckets; i++) {
        if (bucketSize(i) > largestBucketSize) {
            largestBucketSize = bucketSize(i);
            largestBucket = i;
        }
    }
    return largestBucket;
}

template <typename T, unsigned buckets>
unsigned Myhash<T, buckets>::largestBucketSize() const
{
    return bucketSize(largestBucket());
}

#endif
